@@ -121,8 +121,7 @@ glade_xml_init (GladeXML *self)
     priv->tree = NULL;
     priv->tooltips = gtk_tooltips_new();
     gtk_tooltips_enable(priv->tooltips);
-    g_object_ref(priv->tooltips);
-    gtk_object_sink(GTK_OBJECT(priv->tooltips));
+    g_object_ref_sink(priv->tooltips);
     priv->name_hash = g_hash_table_new(g_str_hash, g_str_equal);
     priv->signals = g_hash_table_new(g_str_hash, g_str_equal);
     priv->toplevel = NULL;
@@ -194,6 +193,45 @@ glade_xml_construct (GladeXML *self, const char *fname, const char *root,
     return TRUE;
 }
 
+/**
+ * glade_xml_construct_from_buffer:
+ * @self: the GladeXML object
+ * @buffer: the memory buffer containing the XML document.
+ * @size: the size of the buffer.
+ * @root: the root widget node (or %NULL for none)
+ * @domain: the translation domain (or %NULL for the default)
+ *
+ * In order to use this method you must already have a GladeXML
+ * object.  This two step creation process is typically used only 
+ * by language bindings, such as gtk--, while constructing GladeXML 
+ * objects from compiled-in buffers, at runtime.
+ *
+ * Returns: TRUE if the construction succeeded.
+ */
+gboolean   
+glade_xml_construct_from_buffer (GladeXML *self, const char *buffer, int size,
+					  const char *root,
+					  const char *domain)
+{
+    GladeInterface *iface;
+
+    g_return_val_if_fail(self != NULL, FALSE);
+    g_return_val_if_fail(self->priv->tree == NULL, FALSE);
+
+    iface = glade_parser_parse_buffer(buffer, size, domain);
+
+    if (!iface)
+	return FALSE;
+
+    self->priv->tree = iface;
+    if (self->filename)
+	g_free(self->filename);
+    self->filename = NULL;
+    glade_xml_build_interface(self, iface, root);
+
+    return TRUE;
+}
+
 /**
  * glade_xml_new_from_buffer:
  * @buffer: the memory buffer containing the XML document.
@@ -213,16 +251,14 @@ GladeXML *
 glade_xml_new_from_buffer(const char *buffer, int size, const char *root,
 			  const char *domain)
 {
-    GladeXML *self;
-    GladeInterface *iface = glade_parser_parse_buffer(buffer, size, domain);
-
-    if (!iface)
-	return NULL;
-    self = g_object_new(GLADE_TYPE_XML, NULL);
-
-    self->priv->tree = iface;
-    self->filename = NULL;
-    glade_xml_build_interface(self, iface, root);
+    GladeXML *self = g_object_new(GLADE_TYPE_XML, NULL);
+    self->filename = NULL; /* Not required if g_object_new guarantees filename is NULL.  I do not know if it does. */
+    if( !glade_xml_construct_from_buffer(self, buffer, size, root, domain) )
+    {
+       /* Clean up, the parser was unable to load the object */
+       g_free(self);
+       return NULL;
+    }
 
     return self;
 }
@@ -277,7 +313,7 @@ autoconnect_foreach(const char *signal_handler, GList *signals,
 {
     GCallback func;
 
-    if (!g_module_symbol(allsymbols, signal_handler, (gpointer *)&func))
+    if (!g_module_symbol(allsymbols, signal_handler, (gpointer)&func))
 	g_warning("could not find signal handler '%s'.", signal_handler);
     else
 	for (; signals != NULL; signals = signals->next) {
@@ -368,8 +404,8 @@ autoconnect_full_foreach(const char *signal_handler, GList *signals,
  * @object: the object to connect the signal to.
  * @signal_name: the name of the signal.
  * @signal_data: the string value of the signal data given in the XML file.
- * @connect_object: non NULL if gtk_signal_connect_object should be used.
- * @after: TRUE if the connection should be made with gtk_signal_connect_after.
+ * @connect_object: non NULL if g_signal_connect_object should be used.
+ * @after: TRUE if the connection should be made with g_signal_connect_after.
  * @user_data: the user data argument.
  *
  * This is the signature of a function used to connect signals.  It is used
@@ -776,34 +812,6 @@ glade_xml_add_atk_actions(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info)
     }
 }
 
-/* helper function for adding relations */
-static void
-add_relation(AtkRelationSet *relations, AtkRelationType relation_type,
-	     AtkObject *target_accessible)
-{
-    AtkRelation *relation;
-
-    relation = atk_relation_set_get_relation_by_type (relations,
-						      relation_type);
-    if (relation) {
-	/* add new target accessible to relation */
-	GPtrArray* target_array = atk_relation_get_target (relation);
-	guint i;
-
-	/* first check if target occurs in array ... */
-	for (i = 0; i < target_array->len; i++)
-	    if (g_ptr_array_index(target_array, i) == target_accessible)
-		return;
-	g_object_ref(target_accessible);
-	g_ptr_array_add (target_array, target_accessible);
-    } else {
-	/* the relation hasn't been created yet ... */
-	relation = atk_relation_new(&target_accessible, 1, relation_type);
-	atk_relation_set_add (relations, relation);
-	g_object_unref(relation);
-    }
-}
-
 /* this is a private function */
 static void
 glade_xml_add_atk_relations(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info)
@@ -829,7 +837,8 @@ glade_xml_add_atk_relations(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info)
 
 	    target_accessible = gtk_widget_get_accessible (target_widget);
 
-	    add_relation(relations, relation_type, target_accessible);
+	    atk_relation_set_add_relation_by_type (relations, relation_type,
+						   target_accessible);
 	} else {
 	    GladeDeferredProperty *dprop = g_new(GladeDeferredProperty, 1);
 
@@ -1107,7 +1116,7 @@ guint
 glade_flags_from_string (GType type, const char *string)
 {
     GFlagsClass *fclass;
-    gchar *endptr;
+    gchar *endptr, *prevptr;
     guint i, j, ret = 0;
     char *flagstr;
 
@@ -1127,21 +1136,47 @@ glade_flags_from_string (GType type, const char *string)
 	if (eos || flagstr [i] == '|') {
 	    GFlagsValue *fv;
 	    const char  *flag;
+	    gunichar ch;
 
 	    flag = &flagstr [j];
+            endptr = &flagstr [i];
 
 	    if (!eos) {
 		flagstr [i++] = '\0';
 		j = i;
 	    }
 
-	    fv = g_flags_get_value_by_name (fclass, flag);
+            /* trim spaces */
+	    for (;;)
+	      {
+		ch = g_utf8_get_char (flag);
+		if (!g_unichar_isspace (ch))
+		  break;
+		flag = g_utf8_next_char (flag);
+	      }
 
-	    if (!fv)
-		fv = g_flags_get_value_by_nick (fclass, flag);
+	    while (endptr > flag)
+	      {
+		prevptr = g_utf8_prev_char (endptr);
+		ch = g_utf8_get_char (prevptr);
+		if (!g_unichar_isspace (ch))
+		  break;
+		endptr = prevptr;
+	      }
 
-	    if (fv)
-		ret |= fv->value;
+	    if (endptr > flag)
+	      {
+		*endptr = '\0';
+		fv = g_flags_get_value_by_name (fclass, flag);
+
+		if (!fv)
+		  fv = g_flags_get_value_by_nick (fclass, flag);
+
+		if (fv)
+		  ret |= fv->value;
+		else
+		  g_warning ("Unknown flag: '%s'", flag);
+	      }
 
 	    if (eos)
 		break;
@@ -1193,21 +1228,21 @@ struct _GladeWidgetBuildData {
 /**
  * glade_register_widget:
  * @type: the GType of the widget.
- * @new: the function used to construct instances of the widget.
+ * @new_func: the function used to construct instances of the widget.
  * @build_children: function used to construct children (or NULL).
  * @find_internal_child: function to find internal children (or NULL).
  *
  * This function is used to register new construction functions for a
  * widget type.  The child building routine would call
- * glade_xml_build_widget on each child node to create the child
+ * glade_xml_build_widget() on each child node to create the child
  * before packing it.
  *
  * This function is mainly useful for addon widget modules for libglade
- * (it would get called from the glade_init_module function).
+ * (it would get called from the glade_init_module() function).
  */
 void
 glade_register_widget(GType type,
-		      GladeNewFunc new,
+		      GladeNewFunc new_func,
 		      GladeBuildChildrenFunc build_children,
 		      GladeFindInternalChildFunc find_internal_child)
 {
@@ -1219,11 +1254,11 @@ glade_register_widget(GType type,
     if (glade_build_data_id == 0)
 	glade_build_data_id = g_quark_from_static_string(glade_build_data_key);
 
-    if (!new) new = glade_standard_build_widget;
+    if (!new_func) new_func = glade_standard_build_widget;
 
     data = g_new(GladeWidgetBuildData, 1);
 
-    data->new = new;
+    data->new = new_func;
     data->build_children = build_children;
     data->find_internal_child = find_internal_child;
 
@@ -1442,7 +1477,7 @@ glade_xml_set_value_from_string (GladeXML *xml,
 				 GValue *value)
 {
     GType prop_type;
-    gboolean ret = TRUE;
+    gboolean ret = TRUE, showerr = TRUE;
 
     prop_type = G_PARAM_SPEC_VALUE_TYPE(pspec);
     g_value_init(value, prop_type);
@@ -1498,6 +1533,9 @@ glade_xml_set_value_from_string (GladeXML *xml,
 		g_warning ("could not parse colour name `%s'", string);
 		ret = FALSE;
 	    }
+	} else if (G_VALUE_HOLDS(value, G_TYPE_STRV)) {
+	    char **vector = g_strsplit (string, "\n", 0);
+	    g_value_take_boxed (value, vector);
 	} else
 	    ret = FALSE;
 	break;
@@ -1515,7 +1553,8 @@ glade_xml_set_value_from_string (GladeXML *xml,
 	    adj->page_size = g_strtod(ptr, &ptr);
 
 	    g_value_set_object(value, adj);
-	    gtk_object_sink(GTK_OBJECT(adj));
+	    g_object_ref_sink(adj);
+	    g_object_unref(adj);
 	} else if (G_VALUE_HOLDS(value, GDK_TYPE_PIXBUF)) {
 	    gchar *filename;
 	    GError *error = NULL;
@@ -1532,6 +1571,16 @@ glade_xml_set_value_from_string (GladeXML *xml,
 		ret = FALSE;
 	    }
 	    g_free(filename);
+	} else if (g_type_is_a(GTK_TYPE_WIDGET, G_PARAM_SPEC_VALUE_TYPE(pspec)) ||
+		   g_type_is_a(G_PARAM_SPEC_VALUE_TYPE(pspec), GTK_TYPE_WIDGET)) {
+	    GtkWidget *widget = g_hash_table_lookup(xml->priv->name_hash,
+						    string);
+	    if (widget) {
+		g_value_set_object(value, widget);
+	    } else {
+		ret = FALSE;
+		showerr = FALSE;
+	    }
 	} else
 	    ret = FALSE;
 	break;
@@ -1541,8 +1590,10 @@ glade_xml_set_value_from_string (GladeXML *xml,
     }
 
     if (!ret) {
-	g_warning("could not convert string to type `%s' for property `%s'",
-		  g_type_name(prop_type), pspec->name);
+	if (showerr)
+	    g_warning("could not convert string to type "
+		      "`%s' for property `%s'",
+		      g_type_name(prop_type), pspec->name);
 	g_value_unset(value);
     }
     return ret;
@@ -1621,25 +1672,17 @@ glade_standard_build_widget(GladeXML *xml, GType widget_type,
 	    continue;
 	}
 
-	/* this should catch all properties wanting a GtkWidget
-         * subclass.  We also look for types that could hold a
-         * GtkWidget in order to catch things like the
-         * GtkAccelLabel::accel_object property.  Since we don't do
-         * any handling of GObject or GtkObject directly in
-         * glade_xml_set_value_from_string, this shouldn't be a
-         * problem. */
-	if (g_type_is_a(GTK_TYPE_WIDGET, G_PARAM_SPEC_VALUE_TYPE(pspec)) ||
-	    g_type_is_a(G_PARAM_SPEC_VALUE_TYPE(pspec), GTK_TYPE_WIDGET)) {
-	    deferred_props = g_list_prepend(deferred_props,
-					    &info->properties[i]);
-	    continue;
-	}
-
 	if (glade_xml_set_value_from_string(xml, pspec,
 					    info->properties[i].value,
 					    &param.value)) {
 	    param.name = info->properties[i].name;
 	    g_array_append_val(props_array, param);
+	} else if (g_type_is_a(GTK_TYPE_WIDGET, G_PARAM_SPEC_VALUE_TYPE(pspec)) ||
+		   g_type_is_a(G_PARAM_SPEC_VALUE_TYPE(pspec), GTK_TYPE_WIDGET)) {
+	    /* if the pspec could hold a widget, then try to handle it
+	     * later */
+	    deferred_props = g_list_prepend(deferred_props,
+					    &info->properties[i]);
 	}
     }
     widget = g_object_newv(widget_type, props_array->len,
@@ -1667,7 +1710,7 @@ glade_standard_build_widget(GladeXML *xml, GType widget_type,
 
 	glade_xml_handle_widget_prop(xml, widget, prop->name, prop->value);
     }
-    g_list_free(tmp);
+    g_list_free(deferred_props);
 
     g_array_set_size(props_array, 0);
     g_array_set_size(custom_props_array, 0);
@@ -2046,8 +2089,9 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
 		{
 		    AtkObject *target = gtk_widget_get_accessible(widget);
 
-		    add_relation(dprop->d.rel.relation_set,
-				 dprop->d.rel.relation_type, target);
+		    atk_relation_set_add_relation_by_type
+			(dprop->d.rel.relation_set,
+			 dprop->d.rel.relation_type, target);
 		    g_object_unref(dprop->d.rel.relation_set);
 		}
 		break;
